;**********************************
; "CPM" emulation code for AVSIM  *
;  8085 version                   *
;  rps -- 23 June 87              *
;**********************************
;
; ==================================================================
;NOTES:
;   	1) This is not, in any way, CP/M, but rather a way
;	(I hope) to emulate it on AVSIM.
;
;	2) CP/M and CP/M-like systems differ in what is returned
;	in "duplicate" registers.  This is "standard".  (I hope!)
;
;	3) You must change "MEMSIZE" (below) to fit
;	the CP/M system you are emulating
;
;	4) You must attach AVSIM I/O disk files to the proper
;	memory locations for each function that you will
; 	use:
;		SYMBOL	DIRECTION  FUNCTION
;		kybdin	in         keyboard character in
;		coutpt  out        terminal (CRT, maybe?) out
;		ptr	out	   printer
;		pch	out        paper tape punch
;		rdr	in         paper tape reader
;		kbufin	in	   keyboard buffer line input --
;				   (reads from file until CR)
;
;       5) Set all memory to RAM.
;
;	6) You must assemble and link this with "AVMACZ80 CPM".
;	Then link it with "AVLINK CPM=CPM -SY". 
;       Load the other program into AVSIM with "L D program.HEX"
;       and "L S program.SYM".
;	Then load this code into AVSIM with "L D CPM.HEX",
;	and symbols with "L S CPM.SYM".
;       Note we are loading as "data", NOT ROM.  This is because
;       we want to keep all memory as RAM, as in CPM (many
;       programs DS storage within themselves).
;
;	7) Put a breakpoint on "bkpt" to trap on rest or end 
;       of program, breakpoint "disk" to trap disk access.
;
;	8) Don't forget to have your program setup space for 
;       a stack and set the SP (if you are workig in assembler).
;       (I think that CP/M sets a default stack at 0100h.
;       This program sets up memory there, but does NOT set 
;       the SP.)
;
; ==================================================================
;
;
	PUBLIC kybdin,coutpt,ptr,pch,rdr,kbufin,bkpt,disk
;
MEMSIZE	EQU	1* 1024       ;your choice!

	DEFSEG LOWCPM, START=0000H, CLASS=DATA
	SEG LOWCPM

bkpt:	JMP	0100h		;re-start program on reset

IOBYTE:	DB	00000000B	;set IOBYTE here if needed
LOGIN:	DB 	00H		;logged in drive & user number
TOPMEM: JMP	BDOS		;top of avail. memory (top TPA)
	DS	30H		;reserved (intr. vectors, etc.)
RST7:	DS 	03H		;RST7 -- used by DDT
	DS 	05H		;reserved (intr. vectors)
	DS	10H		;CBIOS scratch area
	DS	0CH		;not used
FCB:	DS	21H		;default file control block
	DS	03h		;default random record pos.
	DS	80H		;DMA buffer area



	DEFSEG HIGHCPM,START=0000H
	SEG HIGHCPM
BDOS:
	JMP	BDOS2

	ORG 2100H
BIOS:
BOOT:	JMP 	0100H		;reset by going to start of program	
WBOOT:  JMP 	0100H		;well, we'll do the same as BOOT
CONST:  JMP 	KBSTAT		;actually, we'll be always ready
CONIN:	JMP 	KBIN		;we'll get char from file
CONOUT: JMP 	CRTOUT		;we'll write to file
LIST:   JMP 	PRNTR		;again, to file
PUNCH:  JMP 	PNCH		;again, to file (does anyone use this?)
READER:	JMP 	READR		;from file
HOME:	JMP 	NULL		;ignore from here on -- at least for now
SELDSK: JMP 	NULL
SETTRK: JMP 	NULL
SETSEC: JMP 	NULL
SETDMA: JMP 	NULL
READ:	JMP 	NULL
WRITE:  JMP 	NULL
LISTST: JMP 	NULL
SECTRAN: JMP 	NULL

KBSTAT:
	MVI 	A,0FFH		;data always ready
	RET

kybdin: DS	01H		;keyboard "buffer"
KBIN:
	LXI	H,kybdin	;connect input file to "kybdin!"
	MOV     A,M
	RET

coutpt: DS	01H		;crt "buffer"
CRTOUT:
	MOV	A,C		;connect output file to "coutpt"!!
	STA	coutpt		
	RET

ptr: DS	01H			;printer "buffer"
PRNTR:
	MOV	A,C		;connect output file to "ptr"!!
	STA	ptr	
	RET

pch: 	DS	01H		;punch "buffer"
PNCH:
	MOV	A,C		;connect output file to "pch"!!
	STA	pch	
	RET

rdr: 	DS	01H		;tape reader "buffer"
READR:
	LXI	H,rdr
	MOV 	A,M		;connect input file to "rdr"!!
	RET

NULL:
disk:	RET			;for now, do nothing.



BDOS2:
	MOV	A,C
	CPI 	00H
	JZ	F00
	CPI 	01H
	JZ	F01
	CPI 	02H
	JZ	F02
	CPI 	03H
	JZ	F03
	CPI 	04H
	JZ	F04
	CPI 	05H
	JZ	F05
	CPI 	06H
	JZ	F06
	CPI 	07H
	JZ	F07
	CPI 	08H
	JZ	F08
	CPI 	09H
	JZ	F09
	CPI 	0AH
	JZ	F0A
	CPI 	0BH
	JZ	F0B

	RET			;just go back if disk call
	
F00:
	LXI	H,0000H		;reset - just start over
	DAD	SP		;reset - just start over
	MVI	M,0000H
	RET			

F01:
	CALL	CONIN		;console read
	MOV	L,A
	MVI 	H,00H
	RET

F02: 
	MOV	C,E		;console out
	CALL 	CONOUT
	RET

F03:
	CALL	READER		;reader read
	MOV	L,A
	MVI	H,00H
	RET

F04: 
	MOV	C,E		;punch out
	CALL 	PUNCH
	RET

F05: 
	MOV	C,E		;list out
	CALL 	LIST
	RET

F06:
	MOV	A,E		;direct console I/O
 	CPI	0FFH
	JZ	F06I		;wants to input
	MOV	C,E		;output
	CALL    CONOUT
	RET
F06I:
	CALL	CONIN		;input
	MOV	L,A
	MVI 	H,00H
	RET

F07:
	LDA	IOBYTE	;get iobyte
	MVI	H,00H
	MOV	L,A
	RET

F08:
	MOV	A,E		;set iobyte
	STA	IOBYTE		
	RET

F09:
	LDAX	D		;print string
	CPI	'$'
	JZ	F091
	MOV	C,A
	CALL	CONOUT
	INX	D
	JMP	F09
F091:	RET


kbufin: DS	01H		;attach I/O file here if you use this!
F0A:				;cons buffr read (reads file to CR)
	MVI	L,00H		;counter
F0A1:	LDA	kbufin
	CPI 	0DH
	JZ	F0A2
	CPI	0AH
	JZ	F0A1		;ignore LF
	STAX	D
	INX	D
	INR	L
	JMP	F0A1
F0A2:	
	MOV	A,L
	MVI	H,00H
	RET

F0B:
	CALL	CONST		;get console status
	MOV	L,A
	MVI	H,00H
	RET

	end


